/* -----------------------------------------------------------------------------
 *    ___   __    __       _      __              __   __
 *   / _ | / /__ / /____ _| | /| / /__  ___  ____/ /  / /____ ____
 *  / __ |/  '_//  '_/ // / |/ |/ / _ `/ -_)/ __/ _ \/ __/ -_) __/
 * /_/ |_/_/\_\/_/\_\\_,_/|__/|__/\_,_/\__/ \__/_//_/\__/\__/_/
 *
 * 080824-1 (c)2009 Elektor International Media B.V.
 * -------------------------------------------------------------------------- */
/* -----------------------------------------------------------------------------
 * This file contains the source code of the function library to control
 * the external ADC over SPI and some function to handle the serial communication
 * -------------------------------------------------------------------------- */

/* -----------------------------------------------------------------------------
 * Include
 * -------------------------------------------------------------------------- */
#include "LPC2103.h"
#include "tools.h"
#include "common.h"



/* -----------------------------------------------------------------------------
 * Define
 * -------------------------------------------------------------------------- */
// MSP3550 Chip Select is on GPIO P0.7
#define MCP3550_CS    0x00000080

// Error LED is on GPIO P0.8
#define LED_ERROR_PIN 0x00000100

// PWM control values
#define PWM_MAX_VALUE 2000




/* -----------------------------------------------------------------------------
 * Private Functions
 * -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/**@author Antoine Authier
 * @date   20090306
 * @brief  set the CS chip select pin of the MCP3550
 * @param[in] mode is HIGH or LOW
 */
static void
MCP3550_chip_select (unsigned char mode)
{
	if (mode == LOW)
	{
		// set external ADC Chip Select low (enable the sampling)
		IOCLR = MCP3550_CS;
	}
	else
	{
		// set external ADC Chip Select high (disable the sampling)
		IOSET = MCP3550_CS;
	}
}

/* -------------------------------------------------------------------------- */
/**@author Antoine Authier
 * @date   20090306
 * @brief  request 8 bit of data from the MCP3550 using the SPI0 bus
 * @return the byte of data from MCP3550
 */
static unsigned char
MCP3550_SPI_get_byte (void)
{
	// send 8 bits to the MCP3550
	S0SPDR = 0;
	// wait until the 'SPI0 transfer complete flag' is set
	while (!(S0SPSR & 0x80));

	// retun the received data
	return(S0SPDR);
}



/* -----------------------------------------------------------------------------
 * Public Functions
 * -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @brief  initialize peripheral SPI0 to enable the communication with the
 *         external ADC.
 */
void
MCP3550_initilize (void)
{
	// set P0.7 the pin for external ADC CS as output
	IODIR |= MCP3550_CS;

	// disable the external ADC sampling
	MCP3550_chip_select(HIGH);

	// select functions: SCK0 (SPI0) and MISO0 (SPI0)
	PINSEL0 |= (1 << 8) | (1 << 10);

	// SPI0 sends and receive 8bits per data transfer, MSB first, no interrupts
	// Data is sampled on the first clock edge of SCK. Transfer starts with first
	// clock edge and ends with the last sampling edge
	// SCK is active LOW
	// the SPI0 operates in Master mode
	S0SPCR = (1 << 3) | (1 << 4) | (1 << 5);

	// set SPI0 clock frequency: SCK0 = PCLK/S0SPCCR
	S0SPCCR = 20;
}

/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @brief  ask a value to the MCP3550
 * @return the value of the ADC convertion
 * @note   bit22 and bit23 of the result are the Overflow Flags and must be
 *         taken in consideration in the application layer.
 */
unsigned long
MCP3550_get_value (void)
{
	unsigned char val1, val2, val3;
	unsigned long value = 0;

	// enable the external ADC sampling
	MCP3550_chip_select(LOW);

// select function MISO0 for SPI0
//TODO check if really necessary	PINSEL0 |= (0 << 10);
	// wait for the sampling and data convertion to be complete
	while (IOPIN & (1 << 5));
// select function MISO0 for SPI0
//TODO check if really necessary	PINSEL0 |= (0 << 10);

	// get the 24 bits value in three time
	val1 = MCP3550_SPI_get_byte();
	val2 = MCP3550_SPI_get_byte();
	val3 = MCP3550_SPI_get_byte();
	// calculate the ADC convertion valus
	value = (val1 << 16) | (val2 << 8) | val3;

	// disable the external ADC sampling
	MCP3550_chip_select(HIGH);

	return(value);
}

/* -------------------------------------------------------------------------- */
/**@author Antoine Authier
 * @date   20090306
 * @brief  initialize hardware timers for blocking timer operations
 * @note   This function MUST BE called at the very beginning of user code
 *         preferably in the main function or just after PLL initialisation
 *         if available (this is not the case in the Akkuwachter).
 *       - Timer0 will be used as the milli-second timer
 *       - Timer1 will be used as the micro-second timer
 */
void
delay_initialize (void)
{
	// set Timer0 prescaler for Timer counter to increase every 1ms
	T0PR = (CPU_CLOCK) / 1000;
	// enable Timer 0 counter and prescaler
	T0TCR = 1;

	// set Timer1 prescaler for Timer counter to increase every 1us
	T1PR = (CPU_CLOCK) / 1000000;
	// enable Timer 1 counter and prescaler
	T1TCR = 1;
}

/* -------------------------------------------------------------------------- */
/**@author Steffen Graf / Antoine Authier
 * @date   20090306
 * @brief  active delay of [us] micro seconds
 * @param[in] delay is the time to wait in micro seconds
 */
void
delay_us (delay_us_t delay)
{
	T1TC = 0;
	while (T1TC < delay);
}

/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @brief  initialize the UART0 serial controller block for serial connection
 * @param[in] baudrate is the communication baud rate
 */
void
UART_initialize (unsigned int baudrate)
{
	// select function TXD0 (UART0) and RXD0 (UART0)
	PINSEL0 |= 0x01 | 0x04;

	// enable and clear Rx and Tx FIFO
	U0FCR = 0x01 | 0x02 | 0x04;
	// serial port config. is 8N1: 8bit No parity and 1 stop bit
	U0LCR = 0x03;
	// enable access to divisor latches
	U0LCR |= 0x80;
	// set laches and dividers to match the desired baud rate
	// see chapter 3.4 of the LPC2103 user manual for calculation
	// here we use a simplify version that may not work...
	U0DLM = 0;
	U0DLL = (unsigned char) (CPU_CLOCK / (baudrate*16));
	// disable access to divisor latches
	U0LCR &= 0x7F;
}

/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @param[in] byte is the character to be sent through the serial port
 * @brief  send one character through the serial port
 */
void
UART0_SendByte (unsigned char byte)
{
	while (!(U0LSR & 0x20));
	U0THR = byte;
}

/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @param[in] string is the character to be sent through the serial port
 * @param[in] end_of_line describe the line termination
 * @brief  senf a string through the serial port
 * @note   the string must be NULL terminated and you can specified the
 *         termination as Line Feed, Carriage Return or Both using the correct
 *         defines. Please note that the string will be 255 character MAX!
 */
void
UART0_SendText (char string[], char end_of_line)
{
	unsigned char i = 0;

	// send each character until the end of the string
	while (string[i] != 0x00)
	{
		UART0_SendByte(string[i]);
		i++;
	}

	// send Line Feed if required
	if (end_of_line & EOL_LINE_FEED)
		UART0_SendByte(ASCII_LF);

	// send Carriage Return if required
	if (end_of_line & EOL_CARRIAGE_RETURN)
		UART0_SendByte(ASCII_CR);
}

/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @param[in] value to tranfer on the serial port
 * @brief  send the decimal representation of value through the serial port 4 digits
 */
void
UART0_SendInt4 (signed short value)
{
	unsigned char byte;

	// send the thousands
	byte = value / 1000;
	value = value - byte * 1000;
	UART0_SendByte('0' + byte);

	// send the hundreds
	byte = value / 100;
	value = value - byte * 100;
	UART0_SendByte('0' + byte);

	// send the tens
	byte = value / 10;
	value = value - byte * 10;
	UART0_SendByte('0' + byte);

	// send the units
	UART0_SendByte('0' + value);
}

/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @param[in] value to tranfer on the serial port
 * @brief  send the decimal representation of value through the serial port 5 digits
 */
void
UART0_SendInt5 (unsigned short value)
{
	unsigned char byte;

	// send the ten-thousands
	byte = value / 10000;
	value = value - byte * 10000;
	UART0_SendByte('0' + byte);

	// send the thousands
	byte = value / 1000;
	value = value - byte * 1000;
	UART0_SendByte('0' + byte);

	// send the hundreds
	byte = value / 100;
	value = value - byte * 100;
	UART0_SendByte('0' + byte);

	// send the tens
	byte = value / 10;
	value = value - byte * 10;
	UART0_SendByte('0' + byte);

	// send the units
	UART0_SendByte('0' + value);
}
/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @param[in] value to tranfer on the serial port
 * @brief  send the decimal representation of value through the serial port 5 digits
 */
void
UART0_SendInt5_signed (signed short value)
{
	if (value >= 0){
		UART0_SendInt5(value);
	}else{
		UART0_SendByte('-');
		value = -value;
		UART0_SendInt5(value);
	}	
}

/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @brief  sample and convert the voltage on AD0.0
 * @return the converted value on 10 bits
 */
unsigned short
ADC_get_AD0_value (void)
{
	// enable AD0.0 on P0.22
	PINSEL1 |= 0x00003000;

	// set the A/D control register to configure the analog to digital convertion
	// - select AD0.0 to be sampled and converted
	// - A/D convertion clock set to PCLK/(val+1) = 2.5MHz
	// - A/D is operational
	// - start convertion now!
	AD0CR = 0x00000001 + 0x00000300 + 0x00200000 + 0x01000000;

	// wait until the conversation is complete (flag done is set)
	while (!(AD0GDR & 0x80000000));

	// return the value of the convertion for AD0.0
	return((unsigned short)((AD0GDR >> 6) & 0x03FF));
}



/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @brief  initialize PWM module on P0.9/MAT2.2 for the LCD backlight
 * @note   this function must be executed prior to any other LCD backlight
 *         related functions
 */
void
LCD_backlight_init (void)
{
	// P0.9 will provide MAT2.2 (Timer 2)
	PINSEL0 |= (1<<19);
	// initialize Timer 2 counter
	T2TC = 0;
	// set Timer 2 prescaler to 0 = no prescaler
	T2PR = 0;
	// TC will reset if MR3 matches TC
	T2MCR = (1<<10);
	// set toggling value
	T2MR2 = PWM_MAX_VALUE;
	// set PWM maximum counter value
	T2MR3 = PWM_MAX_VALUE;
	// enable PWM mode for MATn.2
	T2PWMCON |= (1<<2);
	// When match occurs between TC and MR2 the External Match 2 pin is set,
	// so when connected pin MATn.2 is HIGH
	T2EMR |= (1<<2) | (1<<9);
	// enable Timer 2 Timer Counter and Prescaler Counter for counting
	T2TCR = 1;
}

/* -------------------------------------------------------------------------- */
/**@author Steffen Graf
 * @date   20090306
 * @brief  set LCD backlight level (PWM duty cicle)
 * @param[in] duty_cycle the duty cycle in percent, range [0..100]
 */
void
LCD_backlight_set_cycle (char duty_cycle)
{
	T2MR2 = PWM_MAX_VALUE - (duty_cycle*(PWM_MAX_VALUE/100));
}

/* -------------------------------------------------------------------------- */
/**@author Antoine Authier
 * @date   20090306
 * @brief  initialize the Error LED hardware connection
 */
void
LED_error_initialize (void)
{
	// setup Error LED pin as output GPIO
	PINSEL0 &= 0xFFFCFFFF;
	IODIR |= LED_ERROR_PIN;
	// switch the Error LED off
	LED_error_set_status(LED_OFF);
}

/* -------------------------------------------------------------------------- */
/**@author Antoine Authier
 * @date   20090306
 * @brief  control the Error LED status
 * @param[in] status is the LED status see LED_ERROR_STATUS
 */
void
LED_error_set_status (char status)
{
	if (status == LED_ON) {
		IOSET = LED_ERROR_PIN;
	} else {
		IOCLR = LED_ERROR_PIN;
	}
}
